/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * Main.java
 *
 * Created on Feb 1, 2010, 9:43:24 PM
 */
package filter;

import java.awt.Dimension;
import java.awt.Toolkit;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import serialComms.K2Comms;
import util.Mathematics;

/**
 *
 * @author mm
 */
public class Main extends javax.swing.JPanel {

    K2Comms line;
    int centerX, centerY;

    /** Creates new form Main */
    public Main() {
        initComponents();
        filterPanel.setMain(this);
        enableNr(false);
        enableAn(false);
        updateGains(false, false, false);
        enableDsps(false);
        enableModes(false);
        line = new K2Comms();
        // Get the size of the default screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        // Center it on screen.
        centerX = (int) dim.getWidth() / 2;
        centerY = (int) dim.getHeight() / 2;
    }

    public void setCrystalFilters(FilterModel[][] filters) {
        filterPanel.setCrystalFilters(filters);
        filterPanel.repaint();
    }

    public void setDspFilters(FilterModel[][] filters) {
        filterPanel.setDspFilters(filters);
        filterPanel.repaint();
    }

    public void setCtlVersion(String version) {
        ctlVersion.setText(version);
    }

    public void setDspVersion(String version) {
        dspVersion.setText(version);
    }

    public int getCenterX() {
        return centerX - 568 / 2;
    }

    public int getCenterY() {
        return centerY - 384 / 2;
    }

    public void setupK2() {
        JPopupMenu ports = line.serialPopupChoices(this);
//        System.out.println("centerX = " + centerX + ", Y = " + centerY);
        ports.show(this, centerX / 5, centerY / 5);
    }

    public static void main(String[] args) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                createAndShowGUI();
            }
        });
    }

    /** Make the app's components and display them. */
    private static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("K2 Filter Display");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        Main controller = new Main();
        frame.add(controller);
        frame.pack();
        frame.setVisible(true);
        frame.setLocation(controller.getCenterX(), controller.getCenterY());
        controller.setupK2();
    }

    private void updateGains(boolean cw, boolean ssb, boolean rtty) {
        cwNrText.setEnabled(cw);
        ssbNrText.setEnabled(ssb);
        cwText.setEnabled(cw);
        ssbText.setEnabled(ssb);
        rttyText.setEnabled(rtty);
    }

    public void enableNr(boolean state) {
        if (crystalButton.isSelected()) {
            state = false;
        }
        nrBeta.setEnabled(state);
        nrDecay.setEnabled(state);
        if (state) {
            FilterModel f = filterPanel.getActiveFilter();
            nrBeta.setText(Integer.toString(f.getNrBeta()));
            nrDecay.setText(Integer.toString(f.getNrHold()));
        }
    }

    public void enableDsps(boolean state) {
        dsp1Button.setEnabled(state);
        dsp2Button.setEnabled(state);
        dsp3Button.setEnabled(state);
        dsp4Button.setEnabled(state);
    }

    public void enableAn(boolean state) {
        if (crystalButton.isSelected()) {
            state = false;
        }
        anBeta.setEnabled(state);
        anDecay.setEnabled(state);
        if (state) {
            FilterModel f = filterPanel.getActiveFilter();
            anBeta.setText(Double.toString(f.getNtBeta()));
            anDecay.setText(Double.toString(f.getNtHold()));
        }
    }

    public void enableModes(boolean state) {
        bypassButton.setEnabled(state);
        crystalButton.setEnabled(state);
        cwButton.setEnabled(state);
        cwRevButton.setEnabled(state);
        engageButton.setEnabled(state);
        lsbButton.setEnabled(state);
        usbButton.setEnabled(state);
        rttyButton.setEnabled(state);
        rttyRevButton.setEnabled(state);
    }

    public void updateFilter() {
        filterPanel.setCrystalMode(false);
        if (crystalButton.isSelected()) {
            filterPanel.setCrystalMode(true);
            filterPanel.setChosenDsp(-1);
            enableAn(false);
            enableNr(false);
            updateGains(false, false, false);
            return;
        }
        if (dsp1Button.isSelected()) {
            filterPanel.setChosenDsp(0);
        } else if (dsp2Button.isSelected()) {
            filterPanel.setChosenDsp(1);
        } else if (dsp3Button.isSelected()) {
            filterPanel.setChosenDsp(2);
        } else if (dsp4Button.isSelected()) {
            filterPanel.setChosenDsp(3);
        }
        if (rttyButton.isSelected() || rttyRevButton.isSelected()) {
            enableAn(false);
            enableNr(false);
            updateGains(false, false, true);
        } else if (lsbButton.isSelected() || usbButton.isSelected()) {
            FilterModel f = filterPanel.getActiveFilter();
            enableAn(f.isNotchFilter());
            enableNr(f.getNoiseReductionLevel() > 0);
            updateGains(false, true, false);
        } else if (cwButton.isSelected() || cwRevButton.isSelected()) {
            FilterModel f = filterPanel.getActiveFilter();
            enableAn(false);
            enableNr(f.getNoiseReductionLevel() > 0);
            updateGains(true, false, false);
        }
    }

    public void updateMode() {
        // Update filterPanel's model of filters.
        if (cwButton.isSelected()) {
            filterPanel.setModeCw();
        } else if (cwRevButton.isSelected()) {
            filterPanel.setModeCwRev();
        } else if (lsbButton.isSelected()) {
            filterPanel.setModeLsb();
        } else if (usbButton.isSelected()) {
            filterPanel.setModeUsb();
        } else if (rttyButton.isSelected()) {
            filterPanel.setModeRtty();
        } else if (rttyRevButton.isSelected()) {
            filterPanel.setModeRttyRev();
        }
        // Update UI.
        if (!crystalButton.isSelected()) {
            FilterModel filter = filterPanel.getActiveFilter();
            if (cwButton.isSelected() || cwRevButton.isSelected()) {
                updateGains(true, false, false);
                enableAn(false);
                enableNr(filter.getNoiseReductionLevel() > 0);
            } else if (lsbButton.isSelected() || usbButton.isSelected()) {
                updateGains(false, true, false);
                enableAn(filter.isNotchFilter());
                enableNr(filter.getNoiseReductionLevel() > 0);
            } else if (rttyButton.isSelected() || rttyRevButton.isSelected()) {
                updateGains(false, false, true);
                enableAn(false);
                enableNr(false);
            }
        }
    }

    public void setModeDsp() {
        int mode = filterPanel.getChosenMode();
        line.setMode(mode);
        // Set K2 crystal.
        line.getToDspMenu();
        FilterModel filter = filterPanel.getActiveFilter();
        int dspFil = filterPanel.getChosenDsp();
        if (filterPanel.isModeCw()) {
            line.updateDspFilter(mode, dspFil);
        } else if (filterPanel.isModeRtty()) {
            line.updateDspFilter(mode, dspFil);
        } else if (filterPanel.isModeSsb()) {
            line.updateDspFilter(mode, dspFil);
            if (filter.isNotchFilter()) {
                line.chooseNt();
            }
        }
    }

    public void setModeXtal() {
        int mode = filterPanel.getChosenMode();
        line.setMode(mode);
        // Set K2 crystal.
        int k2xtal = 1 + filterPanel.getChosenCrystal(mode);
        line.useCrystalFilter(k2xtal);
    }

    private int adjustSetting(JTextField tf, String k2MenuName, int whichNum,
            int curVal, int defaultVal, String errorMessage) {
        int newVal = 0;
        try {
            newVal = Integer.valueOf(tf.getText());
        } catch (NumberFormatException n) {
            tf.setText(Integer.toString(defaultVal));
            JOptionPane.showMessageDialog(this,
                    errorMessage, "Input error", JOptionPane.ERROR_MESSAGE);
            return -1;
        }
        if (newVal < 0 || newVal > 99) {
            tf.setText(Integer.toString(defaultVal));
            JOptionPane.showMessageDialog(this,
                    errorMessage, "Input error", JOptionPane.ERROR_MESSAGE);
            return -1;
        }
        int up = newVal - curVal;
        if (up == 0) {
            return -1;
        }
        // Move to KDSP2's "nr # #" menu item.
        boolean there = line.cycleToDspString(k2MenuName, 0, true);
        if (!there) {
            System.out.println(getClass()
                    + ".adjustSetting: couldn't find " + k2MenuName + ".");
            return -1;
        }
        if (whichNum == 1) {
            line.chooseFirstNum();
        } else if (whichNum == 2) {
            line.chooseSecondNum();
        } else {
            System.out.println(getClass() + ".adjustSetting: whichNum is "
                    + whichNum + ", but 1 or 2 was expected.");
            return -1;
        }
        // Hit BAND+ or BAND- 'up' number of times to make change.
        if (up > 0) {
            line.sendBandUp(up);
        } else {
            line.sendBandDown(-up);
        }
        line.exitDspMenu();
        return newVal;
    }

    private double adjustSetting(JTextField tf, String k2MenuName, int whichNum,
            double curVal, double defaultVal, double step, String errorMessage) {
        double newVal = 0;
        try {
            newVal = Integer.valueOf(tf.getText());
        } catch (NumberFormatException n) {
            tf.setText(Double.toString(defaultVal));
            JOptionPane.showMessageDialog(this,
                    errorMessage, "Input error", JOptionPane.ERROR_MESSAGE);
            return -1;
        }
        if (newVal < 0 || newVal > 99) {
            tf.setText(Double.toString(defaultVal));
            JOptionPane.showMessageDialog(this,
                    errorMessage, "Input error", JOptionPane.ERROR_MESSAGE);
            return -1;
        }
        newVal = Mathematics.round(0.5, newVal);
        int up = (int) Math.round((newVal - curVal) / 0.5);
        if (up == 0) {
            return -1;
        }
        // Move to KDSP2's "nr # #" menu item.
        boolean there = line.cycleToDspString(k2MenuName, 0, true);
        if (!there) {
            System.out.println(getClass()
                    + ".adjustSetting: couldn't find " + k2MenuName + ".");
            return -1;
        }
        if (whichNum == 1) {
            line.chooseFirstNum();
        } else if (whichNum == 2) {
            line.chooseSecondNum();
        } else {
            System.out.println(getClass() + ".adjustSetting: whichNum is "
                    + whichNum + ", but 1 or 2 was expected.");
            return -1;
        }
        // Hit BAND+ or BAND- 'up' number of times to make change.
        if (up > 0) {
            line.sendBandUp(up);
        } else {
            line.sendBandDown(-up);
        }
        line.exitDspMenu();
        return newVal;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        filterButtonGroup = new javax.swing.ButtonGroup();
        modeButtonGroup = new javax.swing.ButtonGroup();
        kdsp2 = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        filterPanel = new filter.FilterPanel();
        jPanel2 = new javax.swing.JPanel();
        crystalButton = new javax.swing.JRadioButton();
        dsp1Button = new javax.swing.JRadioButton();
        dsp2Button = new javax.swing.JRadioButton();
        dsp3Button = new javax.swing.JRadioButton();
        dsp4Button = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        cwButton = new javax.swing.JRadioButton();
        lsbButton = new javax.swing.JRadioButton();
        rttyButton = new javax.swing.JRadioButton();
        rttyRevButton = new javax.swing.JRadioButton();
        usbButton = new javax.swing.JRadioButton();
        cwRevButton = new javax.swing.JRadioButton();
        jPanel4 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        nrBeta = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        nrDecay = new javax.swing.JTextField();
        jPanel6 = new javax.swing.JPanel();
        engageButton = new javax.swing.JRadioButton();
        bypassButton = new javax.swing.JRadioButton();
        anPanel = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        anBeta = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        anDecay = new javax.swing.JTextField();
        jPanel8 = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        cwNrText = new javax.swing.JTextField();
        jLabel8 = new javax.swing.JLabel();
        ssbNrText = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        cwText = new javax.swing.JTextField();
        ssbText = new javax.swing.JTextField();
        rttyText = new javax.swing.JTextField();
        dspVersion = new javax.swing.JLabel();
        ctlVersion = new javax.swing.JLabel();

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("K2 Filter Display"));

        javax.swing.GroupLayout filterPanelLayout = new javax.swing.GroupLayout(filterPanel);
        filterPanel.setLayout(filterPanelLayout);
        filterPanelLayout.setHorizontalGroup(
            filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 525, Short.MAX_VALUE)
        );
        filterPanelLayout.setVerticalGroup(
            filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 146, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(filterPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(filterPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Filter"));

        filterButtonGroup.add(crystalButton);
        crystalButton.setSelected(true);
        crystalButton.setText("Crystal");
        crystalButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterHandler(evt);
            }
        });

        filterButtonGroup.add(dsp1Button);
        dsp1Button.setText("DSP 1");
        dsp1Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterHandler(evt);
            }
        });

        filterButtonGroup.add(dsp2Button);
        dsp2Button.setText("DSP 2");
        dsp2Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterHandler(evt);
            }
        });

        filterButtonGroup.add(dsp3Button);
        dsp3Button.setText("DSP 3");
        dsp3Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterHandler(evt);
            }
        });

        filterButtonGroup.add(dsp4Button);
        dsp4Button.setText("DSP 4");
        dsp4Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(crystalButton)
                    .addComponent(dsp1Button)
                    .addComponent(dsp2Button)
                    .addComponent(dsp3Button)
                    .addComponent(dsp4Button))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(crystalButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(dsp1Button)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(dsp2Button)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(dsp3Button)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(dsp4Button))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Mode"));

        modeButtonGroup.add(cwButton);
        cwButton.setSelected(true);
        cwButton.setText("CW");
        cwButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        modeButtonGroup.add(lsbButton);
        lsbButton.setText("LSB");
        lsbButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        modeButtonGroup.add(rttyButton);
        rttyButton.setText("RTTY");
        rttyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        modeButtonGroup.add(rttyRevButton);
        rttyRevButton.setText("RTTY rev");
        rttyRevButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        modeButtonGroup.add(usbButton);
        usbButton.setText("USB");
        usbButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        modeButtonGroup.add(cwRevButton);
        cwRevButton.setText("CW rev");
        cwRevButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modeHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cwButton)
                    .addComponent(cwRevButton)
                    .addComponent(lsbButton)
                    .addComponent(usbButton)
                    .addComponent(rttyButton)
                    .addComponent(rttyRevButton))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(cwButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cwRevButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lsbButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(usbButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(rttyButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(rttyRevButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Denoiser"));

        jLabel1.setText("Beta:");

        nrBeta.setText("89");
        nrBeta.setToolTipText("Defines how quickly the denoiser adjusts.");
        nrBeta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        jLabel2.setText("Decay:");

        nrDecay.setText("71");
        nrDecay.setToolTipText("Defines how quickly the denoiser abandons adjustments.");
        nrDecay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2))
                .addGap(8, 8, 8)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(nrDecay)
                    .addComponent(nrBeta, javax.swing.GroupLayout.DEFAULT_SIZE, 30, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(nrBeta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(nrDecay, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("KDSP2"));

        kdsp2.add(engageButton);
        engageButton.setSelected(true);
        engageButton.setText("Engage");
        engageButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                kdsp2Handler(evt);
            }
        });

        kdsp2.add(bypassButton);
        bypassButton.setText("Bypass");
        bypassButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                kdsp2Handler(evt);
            }
        });

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(engageButton)
                    .addComponent(bypassButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addComponent(engageButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(bypassButton)
                .addContainerGap(10, Short.MAX_VALUE))
        );

        anPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Autonotch"));

        jLabel5.setText("Beta:");

        anBeta.setText("39");
        anBeta.setToolTipText("Defines how quickly the autonotch adjusts.");
        anBeta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        jLabel6.setText("Decay:");

        anDecay.setText("7");
        anDecay.setToolTipText("Defines how quickly the autonotch abandons adjustments.");
        anDecay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        javax.swing.GroupLayout anPanelLayout = new javax.swing.GroupLayout(anPanel);
        anPanel.setLayout(anPanelLayout);
        anPanelLayout.setHorizontalGroup(
            anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(anPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel5)
                    .addComponent(jLabel6))
                .addGap(8, 8, 8)
                .addGroup(anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(anDecay)
                    .addComponent(anBeta, javax.swing.GroupLayout.DEFAULT_SIZE, 30, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        anPanelLayout.setVerticalGroup(
            anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(anPanelLayout.createSequentialGroup()
                .addGroup(anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(anBeta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(anPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(anDecay, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(15, Short.MAX_VALUE))
        );

        jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder("Gain"));

        jLabel7.setText("CW nr:");

        cwNrText.setText("17.5");
        cwNrText.setToolTipText("The setting may be changed in 0.5 dB steps, from -5.5 to +18.0 dB.");
        cwNrText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        jLabel8.setText("SSB nr:");

        ssbNrText.setText("7");
        ssbNrText.setToolTipText("The setting may be changed in 0.5 dB steps, from -5.5 to +18.0 dB.");
        ssbNrText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        jLabel9.setText("CW:");

        jLabel10.setText("SSB:");

        jLabel11.setText("RTTY:");

        cwText.setText("0");
        cwText.setToolTipText("The setting may be changed in 0.5 dB steps, from -5.5 to +18.0 dB.");
        cwText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        ssbText.setText("7");
        ssbText.setToolTipText("nulThe setting may be changed in 0.5 dB steps, from -5.5 to +18.0 dB.");
        ssbText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        rttyText.setText("7");
        rttyText.setToolTipText("The setting may be changed in 0.5 dB steps, from -5.5 to +18.0 dB.");
        rttyText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textHandler(evt);
            }
        });

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel7)
                            .addComponent(jLabel8))
                        .addGap(8, 8, 8)
                        .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(ssbNrText, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE)
                            .addComponent(cwNrText, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE)))
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addComponent(jLabel9)
                        .addGap(27, 27, 27)
                        .addComponent(cwText, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE))
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addComponent(jLabel10)
                        .addGap(25, 25, 25)
                        .addComponent(ssbText, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE))
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addComponent(jLabel11)
                        .addGap(15, 15, 15)
                        .addComponent(rttyText, javax.swing.GroupLayout.DEFAULT_SIZE, 42, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(cwNrText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(ssbNrText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(cwText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel10)
                    .addComponent(ssbText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel11)
                    .addComponent(rttyText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(27, Short.MAX_VALUE))
        );

        dspVersion.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        dspVersion.setForeground(new java.awt.Color(102, 102, 102));
        dspVersion.setText("   ");

        ctlVersion.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        ctlVersion.setForeground(new java.awt.Color(102, 102, 102));
        ctlVersion.setText("  ");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(8, 8, 8)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(anPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(dspVersion, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(ctlVersion, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, 171, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(dspVersion)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ctlVersion))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(anPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(jPanel8, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void filterHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_filterHandler
        updateFilter();
        updateMode();
        JRadioButton b = (JRadioButton) evt.getSource();
        if (b == crystalButton) {
            setModeXtal();
        } else {
            int uiFilter;
            if (b == dsp1Button) {
                uiFilter = 1;
            } else if (b == dsp2Button) {
                uiFilter = 2;
            } else if (b == dsp3Button) {
                uiFilter = 3;
            } else {
                uiFilter = 4;
            }
            line.chooseDspFilter(uiFilter);
        }
    }//GEN-LAST:event_filterHandler

    private void modeHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_modeHandler
        updateMode();
        filterPanel.repaint();
    }//GEN-LAST:event_modeHandler

    private void kdsp2Handler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_kdsp2Handler
        boolean state = true;
        if (bypassButton.isSelected()) {
            state = false;
            enableNr(false);
            updateGains(false, false, false);
        }
        crystalButton.setSelected(true);
        updateFilter();
        updateMode();
        enableDsps(state);
    }//GEN-LAST:event_kdsp2Handler

    private void textHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textHandler
        JTextField f = (JTextField) evt.getSource();
        // Update K2 with user's changes.
        FilterModel filter = filterPanel.getActiveFilter();
        if (f == nrBeta) {
            int newVal = adjustSetting(f, "NR", 1,
                    filter.getNrBeta(), 89, "Integer from 0 to 99 expected.");
            filterPanel.setNrBeta(newVal);
            filter.setNrBeta(newVal);
        } else if (f == nrDecay) {
            int newVal = adjustSetting(f, "NR", 2,
                    filter.getNrHold(), 71, "Integer from 0 to 99 expected.");
            filterPanel.setNrHold(newVal);
            filter.setNrHold(newVal);
        } else if (f == anBeta) {
            int newVal = adjustSetting(f, "NT", 1,
                    filter.getNrBeta(), 39, "Integer from 0 to 99 expected.");
            filterPanel.setNtBeta(newVal);
            filter.setNtBeta(newVal);
        } else if (f == anDecay) {
            int newVal = adjustSetting(f, "NT", 2,
                    filter.getNtHold(), 7, "Integer from 0 to 99 expected.");
            filterPanel.setNtHold(newVal);
            filter.setNtHold(newVal);
        } else if (f == cwNrText) {
            double newVal = adjustSetting(f, "N G", 2,
                    filter.getDenoiserGain(), 0, 0.5,
                    "Real from -5.5 to 18 in steps of 0.5 dB expected.");
            filterPanel.setCwDenoiser(newVal);
            filter.setDenoiserGain(newVal);
        } else if (f == ssbNrText) {
            double newVal = adjustSetting(f, "N G", 2,
                    filter.getDenoiserGain(), 7.0, 0.5,
                    "Real from -5.5 to 18 in steps of 0.5 dB expected.");
            filterPanel.setSsbDenoiser(newVal);
            filter.setDenoiserGain(newVal);
        } else if (f == cwText || f == ssbText || f == rttyText) {
            double newVal = adjustSetting(f, "F G", 2,
                    filter.getFilterGain(), 0, 0.5,
                    "Real from -5.5 to 18 in steps of 0.5 dB expected.");
            filter.setFilterGain(newVal);
        }
    }//GEN-LAST:event_textHandler
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField anBeta;
    private javax.swing.JTextField anDecay;
    private javax.swing.JPanel anPanel;
    private javax.swing.JRadioButton bypassButton;
    private javax.swing.JRadioButton crystalButton;
    javax.swing.JLabel ctlVersion;
    private javax.swing.JRadioButton cwButton;
    private javax.swing.JTextField cwNrText;
    private javax.swing.JRadioButton cwRevButton;
    private javax.swing.JTextField cwText;
    private javax.swing.JRadioButton dsp1Button;
    private javax.swing.JRadioButton dsp2Button;
    private javax.swing.JRadioButton dsp3Button;
    private javax.swing.JRadioButton dsp4Button;
    private javax.swing.JLabel dspVersion;
    private javax.swing.JRadioButton engageButton;
    private javax.swing.ButtonGroup filterButtonGroup;
    private filter.FilterPanel filterPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel8;
    private javax.swing.ButtonGroup kdsp2;
    private javax.swing.JRadioButton lsbButton;
    private javax.swing.ButtonGroup modeButtonGroup;
    private javax.swing.JTextField nrBeta;
    private javax.swing.JTextField nrDecay;
    private javax.swing.JRadioButton rttyButton;
    private javax.swing.JRadioButton rttyRevButton;
    private javax.swing.JTextField rttyText;
    private javax.swing.JTextField ssbNrText;
    private javax.swing.JTextField ssbText;
    private javax.swing.JRadioButton usbButton;
    // End of variables declaration//GEN-END:variables
}
