/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package serialComms;

import filter.Config;
import filter.FilterModel;
import filter.FilterPanel;
import filter.Main;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import util.Serial;

/**
 *
 * @author mm
 */
public class K2Comms {

    boolean[] decimalPoint;
    DSdecoder dec;
    FilterModel[][] xfil = new FilterModel[6][4]; // 6 modes, 4 filters/mode.
    FilterModel[][] afil = new FilterModel[3][4]; // 3 modes, 4 filters/mode.
    Serial sport;

    public K2Comms() {
        sport = new Serial();
    }

    public boolean initialize() {

        dec = new DSdecoder();
        // Set up K2 in extended command mode.
        sport.serialWrite("K22;", 100); // Use extended command mode.
        sport.serialWrite("K2;", 100); // Verify mode.
        byte[] resp = new byte[32];
        sport.serialRead(resp);
        String r = new String(resp);
        return r.indexOf("K22;") != -1;

        // Verify that a KDSP2 is present.
    }

    public void noiseReduction(boolean turnOn) {
        String s = (turnOn ? "" : ".") + "nr";
        if (cycleToDspString(s, 3, true)) {
            if (turnOn) {
                sendBandUp(1);
            } else {
                sendBandDown(1);
            }
        }
    }

    public void autoNotch(boolean turnOn) {
        String s = (turnOn ? "" : ".") + "nt";
        if (cycleToDspString(s, 3, true)) {
            if (turnOn) {
                sendBandUp(1);
            } else {
                sendBandDown(1);
            }
        }
    }

    public void sendBandUp(int count) {
        for (int i = 0; i < count; i++) {
            sport.serialWrite("SW01;", 100);
        }
    }

    public void sendBandUp(int count, int delay) {
        for (int i = 0; i < count; i++) {
            sport.serialWrite("SW01;", delay);
        }
    }

    public void sendBandDown(int count) {
        for (int i = 0; i < count; i++) {
            sport.serialWrite("SW03;", 100);
        }
    }

    public int getCurrentDspFilter() {
        sport.serialWrite("FW;", 100);
        byte[] b = new byte[16];
        sport.serialRead(b);
        String s = new String(b);
        int semiPos = s.indexOf(';');
        char num = s.charAt(semiPos - 1);
        return num - 48; // Ascii '0' is 48, so convert to integer.
    }

    public boolean getToDspMenu() {
        int max = 10;
        int i = 0;
        sport.serialWrite("SW02;", 300); // DISPLAY, to get into DSP menu.
        String ds = dec.getDisplay(sport, true);
        do {
            char c0 = ds.charAt(0);
            if (!(c0 == 'E' || (c0 >= '0' && c0 <= '9') || c0 == ' ')) {
                // Already in DSP menu.
                return true;
            }
            sport.serialWrite("SW02;", 300); // DISPLAY, to get into DSP menu.
            ds = dec.getDisplay(sport, true);
            i++;
        } while (i < max);
        return false;
    }

    public boolean exitDspMenu() {
        int max = 10;
        int i = 0;
        sport.serialWrite("SW02;", 300); // DISPLAY, to get into DSP menu.
        String ds = dec.getDisplay(sport, true);
        do {
            char c0 = ds.charAt(0);
            if ((c0 >= '0' && c0 <= '9') || c0 == ' ') {
                // Already out of DSP menu.
                return true;
            }
            sport.serialWrite("SW02;", 300); // DISPLAY, to get into DSP menu.
            ds = dec.getDisplay(sport, true);
            i++;
        } while (i < max);
        System.out.println(getClass() + ".exitDspMenu: couldn't exit DSP menu.");
        return false;
    }

    public boolean cycleToDspString(String s, int index, boolean displayDPs) {
        int max = 10;
        int i = 0;
        if (!getToDspMenu()) {
            System.out.println(getClass()
                    + ".cycleToString: couldn't reach DSP menu!");
            return false;
        }
        sport.serialWrite("SW17;", 100); // RCL, to cycle through DSP menus.
        String ds = dec.getDisplay(sport, displayDPs);
        do {
            if (ds.indexOf(s, index) == index) {
                return true;
            }
            sport.serialWrite("SW17;", 100); // RCL, to cycle through DSP menus.
            ds = dec.getDisplay(sport, displayDPs);
            i++;
        } while (i < max);
//        sport.serialWrite("SW02;", 200); // DISPLAY, to get out of DSP menu.
        System.out.println(getClass() + ".cycleToString: couldn't find " + s
                + "!");
        return false;
    }

    public void chooseNr() {
        dec.getDisplay(sport, true);
        boolean[] dp = dec.getDecimalPoints();
        if (dp[6]) {
            sport.serialWrite("SW19;", 200); // STORE, to choose nr
        }
    }

    public void chooseNt() {
        dec.getDisplay(sport, true);
        boolean[] dp = dec.getDecimalPoints();
        if (dp[3]) {
            sport.serialWrite("SW19;", 200); // STORE, to choose nt
        }
    }

    public void chooseDspFilter(int newFilterNum) {
        if (newFilterNum < 1 || newFilterNum > 4) {
            System.out.println(getClass() + ".chooseDspFilter: bad filter, "
                    + newFilterNum + ".  Must be between 1 and 4.");
        }
        int curFilterNum = getCurrentDspFilter();
        int pushes = newFilterNum - curFilterNum;
        if (pushes < 0) {
            pushes += 4;
        }
        for (int i = 0; i < pushes; i++) {
            sport.serialWrite("SW29;", 100); // AFIL, to advance to next xfil.
        }
    }

    public void chooseFirstNum() {
        String display = dec.getDisplay(sport, true);
        String first = display.substring(2, 5);
        String second = display.substring(5, 8);
        sport.serialWrite("SW01;", 100); // BAND+
        display = dec.getDisplay(sport, true);
        String firstP = display.substring(2, 5);
        String secondP = display.substring(5, 8);
        if (first.equals(firstP) && !second.equals(secondP)) {
            // Second number was chosen.
            sport.serialWrite("SW03;", 100); // BAND-
            sport.serialWrite("SW19;", 100); // STORE
        } else if (!first.equals(firstP) && second.equals(secondP)) {
            // First number was chosen.
            sport.serialWrite("SW03;", 100); // BAND-
        } else {
            // Neither number changed, so must be at top of scale.
            sport.serialWrite("SW03;", 100); // BAND-
            display = dec.getDisplay(sport, true);
            firstP = display.substring(2, 5);
            secondP = display.substring(5, 8);
            if (first.equals(firstP) && !second.equals(secondP)) {
                // Second number was chosen.
                sport.serialWrite("SW01;", 100); // BAND+
                sport.serialWrite("SW19;", 100); // STORE
            } else if (!first.equals(firstP) && second.equals(secondP)) {
                // First number was chosen.
                sport.serialWrite("SW01;", 100); // BAND+
            } else {
                System.out.println(getClass() + ".chooseFirstNum: "
                        + "numbers aren't changing!  Giving up.");
                return;
            }
        }
    }

    public void chooseSecondNum() {
        String display = dec.getDisplay(sport, true);
        String first = display.substring(2, 5);
        String second = display.substring(5, 8);
        sport.serialWrite("SW01;", 100); // BAND+
        display = dec.getDisplay(sport, true);
        String firstP = display.substring(2, 5);
        String secondP = display.substring(5, 8);
        if (first.equals(firstP) && !second.equals(secondP)) {
            // Second number was chosen.
            sport.serialWrite("SW03;", 100); // BAND-
        } else if (!first.equals(firstP) && second.equals(secondP)) {
            // First number was chosen.
            sport.serialWrite("SW03;", 100); // BAND-
            sport.serialWrite("SW19;", 100); // STORE
        } else {
            // Neither number changed, so must be at top of scale.
            sport.serialWrite("SW03;", 100); // BAND-
            display = dec.getDisplay(sport, true);
            firstP = display.substring(2, 5);
            secondP = display.substring(5, 8);
            if (first.equals(firstP) && !second.equals(secondP)) {
                // Second number was chosen.
                sport.serialWrite("SW01;", 100); // BAND+
            } else if (!first.equals(firstP) && second.equals(secondP)) {
                // First number was chosen.
                sport.serialWrite("SW01;", 100); // BAND+
                sport.serialWrite("SW19;", 100); // STORE
            } else {
                System.out.println(getClass() + ".chooseSecondNum: "
                        + "numbers aren't changing!  Giving up.");
                return;
            }
        }
    }

    public void sendK2Cmd(String cmd) {
        sport.serialWrite(cmd, 100);
    }

    public void setMode(int mode) {
        int k2mode;
        switch (mode) {
            case 0:
                k2mode = 3; // CW
                break;
            case 1:
                k2mode = 7; // CW rev
                break;
            case 2:
                k2mode = 1; // LSB
                break;
            case 3:
                k2mode = 2; // USB
                break;
            case 4:
                k2mode = 6; // RTTY
                break;
            case 5:
                k2mode = 9; // RTTY rev
                break;
            default:
                System.out.println(getClass()
                        + ".setMode: illegal mode.  Ignoring...");
                return;
        }
        String cmd = String.format("MD%d;", k2mode);
        sport.serialWrite(cmd, 100);
    }

    public void useCrystalFilter(int filterNumber) {
        if (filterNumber < 1 || filterNumber > 4) {
            System.out.println(getClass() + ".useCrystalFilter: crazy filter, "
                    + filterNumber + "!  Ignoring request.");
            return;
        }
        String cmd = String.format("FW0000%d;", filterNumber);
        sport.serialWrite(cmd, 100);
    }

    public void updateDspFilter(int mode, int filter) {
        FilterModel af = afil[mode][filter];
        // DSP filter is already chosen on k2 by prior chooseDspFilter() call.
        String d = dec.getDisplay(sport, true);

        // Update CW narrow/wide filter type.
        if (mode == FilterPanel.CW || mode == FilterPanel.CW_REV) {
            if ((af.isCwNarrow() && d.charAt(0) == 'c')
                    || (!af.isCwNarrow() && d.charAt(0) == 'C')) {
                System.out.println("Toggling widenarrow cw filters");
                sport.serialWrite("SW03;", 100); // BAND-, to toggle setting.
            }
        }

        // Update noise reduction setting.
        boolean nrOn = d.indexOf(".NR") != -1;
        if ((nrOn && af.getNoiseReductionLevel() == 0)
                || (!nrOn && af.getNoiseReductionLevel() > 0)) {
            sport.serialWrite("SW01;", 100); // BAND+, to toggle NR on/off.
        }

        // Update filter freqs.
        sport.serialWrite("SW17;", 100); // RCL, to move to filter display.
        d = dec.getDisplay(sport, false);
        int k2F1 = 0; // Freq read from K2.
        int storedF1; // Freq stored in this program.
        int inc;
        d = dec.getOldDisplay(true);
        if (mode == FilterPanel.LSB || mode == FilterPanel.USB) {
            System.out.println("SSB filter");
            if (d.indexOf("LPASS") != -1) {
                k2F1 = 3000;
            }
            inc = 100;
            chooseSecondNum();
            storedF1 = (int) af.getHighFreqCutoff();
        } else if (mode == FilterPanel.RTTY || mode == FilterPanel.RTTY_REV) {
            System.out.println("RTTY filter");
            if (d.indexOf("LPASS") != -1) {
                k2F1 = 1900;
            }
            inc = 100;
            chooseSecondNum();
            storedF1 = (int) af.getHighFreqCutoff();
        } else {
            System.out.println("CW filter");
            if (d.indexOf("LPASS") != -1) {
                k2F1 = 800;
            }
            inc = 50;
            storedF1 = (int) af.getBandwidth();
        }
        int i = d.lastIndexOf(' ');
        if (k2F1 == 0) {
            k2F1 = (int) (Double.valueOf(d.substring(i + 1)) * 1000);
            System.out.println("DS: '" + d + "'");
            System.out.println("K2F1 = " + k2F1);
        }
        System.out.println("k2F1 = " + k2F1);
        System.out.println("storedF1 = " + storedF1 + " Hz");
        if (k2F1 != storedF1) {
            int taps = (k2F1 - storedF1) / inc;
            System.out.println("inc = " + inc + " Hz");
            System.out.println("f1 #taps = " + taps);
            if (taps > 0) {
                sendBandDown(taps);
            } else if (taps < 0) {
                sendBandUp(-taps, 300);
            }
        }

        int k2F0 = -1;
        int storedF0 = -1;
        if (mode == FilterPanel.CW || mode == FilterPanel.CW_REV) {
            if (d.indexOf("LPASS") != -1) {
                k2F0 = 0;
            }
            inc = 50;
            storedF0 = (int) af.getCenterFreq();
        } else if (mode == FilterPanel.LSB || mode == FilterPanel.USB) {
            if (d.indexOf("LPASS") != -1) {
                k2F0 = 0;
            }
            inc = 100;
            storedF0 = (int) af.getLowFreqCutoff();
        } else if (mode == FilterPanel.RTTY || mode == FilterPanel.RTTY_REV) {
            if (d.indexOf("LPASS") != -1) {
                k2F0 = 0;
            }
            inc = 200;
            storedF0 = (int) af.getLowFreqCutoff();
        }

        if (k2F0 == -1) {
            d = dec.getOldDisplay(true);
            chooseFirstNum();
            i = d.indexOf(' ');
            int j = d.indexOf(' ', i + 1);
            k2F0 = (int) (Double.valueOf(d.substring(i, j)) * 1000);
            System.out.println("DS: '" + d + "'");
            System.out.println("K2F0 = " + k2F0);
        }
        System.out.println("k2F0 = " + k2F0 + " Hz");
        System.out.println("storedF0 = " + storedF0 + " Hz");
        if (k2F0 != storedF0) {
            int taps = (k2F0 - storedF0) / inc;
            System.out.println("inc = " + inc + " Hz");
            System.out.println("f0 #taps = " + taps);
            if (taps > 0) {
                sendBandDown(taps);
            } else if (taps < 0) {
                sendBandUp(-taps, 300);
            }
        }

        // Skip past rest of DSP menus.
        cycleToDspString("NR", 3, false);
        sport.serialWrite("SW02;", 100); // DISPLAY, to exit DSP display.
    }

    public boolean isKdsp2Installled() {
        boolean installed = false;
        return installed;
    }

    public String kdsp2Version() {
        // Send DISPLAY, RCL, RCL, RCL, RCL, STORE, then capture display.
        sport.serialWrite("SW02;", 100); // DISPLAY
        for (int i = 0; i < 6; i++) {
            sport.serialWrite("SW17;", 100); // RCL
            String d = dec.getDisplay(sport, true);
            if (d.length() >= 4 && d.charAt(3) == 'G') {
                break;
            }
        }
        sport.serialWrite("SW19;"); // STORE
        sleep(100);
        String dsp = dec.getDisplay(sport, true);
        sleep(1000);
        String ctl = dec.getDisplay(sport, true);
        sleep(1000);
        sport.serialWrite("SW02;", 200); // DISPLAY
        return ctl;
    }

    public String kdsp2CtlVersion() {
        // Send DISPLAY, RCL, RCL, RCL, RCL, STORE, then capture display.
        sport.serialWrite("SW02;", 100); // DISPLAY
        for (int i = 0; i < 6; i++) {
            sport.serialWrite("SW17;", 100); // RCL
            String d = dec.getDisplay(sport, true);
            if (d.length() >= 4 && d.charAt(3) == 'G') {
                break;
            }
        }
        sport.serialWrite("SW19;"); // STORE
        sleep(1000);
        String dsp = dec.getDisplay(sport, true);
        sleep(2000);
        String ctl = dec.getDisplay(sport, true);
        sleep(1000);
        sport.serialWrite("SW02;", 1000); // DISPLAY
        return ctl;
    }

    public void getK2CrystalFilterSettings(int mode) {
        String f[] = new String[4];
        // Start with xfil FL1.
        f[0] = varVal("FW00001;", true, 300);
        f[1] = varVal("FW0000;", true, 300);
        f[2] = varVal("FW0000;", true, 300);
        f[3] = varVal("FW0000;", true, 300);
        varVal("FW00001;", false);
        System.out.println("XFIL:");
        for (int i = 0; i < 4; i++) {
            System.out.println("XFIL (mode " + mode + ") " + i + " = " + f[i]);
            // Response is like "FL1 1.70c" so strip non-numbers.
            int len = f[i].length();
            int n = f[i].charAt(2) - '0' - 1;
            f[i] = f[i].substring(f[i].indexOf(' ') + 1, len - 1);
            if (f[i].equals("OP1")) {
                f[i] = "2.8"; // kHz
            }
            if (xfil[mode][n] == null) {
                xfil[mode][n] = new FilterModel();
            }
            System.out.println("  bw = " + f[i] + " kHz");
            xfil[mode][n].setBandwidth(1000 * Double.valueOf(f[i]));
        }
    }

    public void getK2AllCrystalSettings() {
        // CW mode
        varVal("MD3;", false);
        getK2CrystalFilterSettings(0);
        // CW reverse mode.
        varVal("MD7;", false);
        getK2CrystalFilterSettings(1);
        // LSB mode.
        varVal("MD1;", false);
        getK2CrystalFilterSettings(2);
        // USB mode.
        varVal("MD2;", false);
        getK2CrystalFilterSettings(3);
        // RTTY mode.
        varVal("MD6;", false);
        getK2CrystalFilterSettings(4);
        // RTTY reverse mode.
        varVal("MD9;", false);
        getK2CrystalFilterSettings(5);
    }

    public void getDSPSettings() {
        String first, s;

        int filter = -1;
        for (int m = 0; m < 3; m++) {
            String cmd = null;
            switch (m) {
                case 0:
                    cmd = "MD3;"; // CW
                    break;
                case 1:
                    cmd = "MD1;"; // SSB
                    break;
                case 2:
                    cmd = "MD6;"; // RTTY
                    break;
            }
            varVal(cmd, false);
            for (int f = 0; f < 4; f++) {
                FilterModel fm = new FilterModel();
                varVal("SW02;", false, 100); // DISPLAY, to get 1st KDSP2 xfil.
                s = dec.getDisplay(sport, true);
                first = new String(s);
                int cycles = 0;
                do {
                    fm.setNoiseReductionLevel(0); // Assume this for now.
                    fm.setCwNarrow(s.charAt(0) == 'C');
                    if (s != null && s.length() > 0) {
                        if (s.charAt(1) >= '1' && s.charAt(1) <= '4') {
                            filter = Integer.valueOf(s.substring(1, 2));
                            if (s.indexOf(".NR") != -1) {
                                // Denoiser is on, find level.
                                varVal("SW01;", false, 100); // DISPLAY, turn off nr.
                                varVal("SW01;", false, 100); // DISPLAY, turn on and get level.
                                s = dec.getDisplay(sport, true);
                            }
                            fm.setNotchFilter(s.indexOf(".NT") != -1);
                            if (s.indexOf("NR") == -1) {
                                // Filter specs.
                                if (s.indexOf("LPA") != -1) {
                                    fm.setLowFreqCutoff(0);
                                    fm.setHighFreqCutoff(3400);
                                    fm.setCenterFreq(1700);
                                    fm.setBandwidth(3400);
                                } else {
                                    double num1 = Double.valueOf(s.substring(3, 6));
                                    int l = s.length();
                                    double num2 = Double.valueOf(
                                            s.substring(l - 3, l));
                                    num1 *= 1000; // Convert kHz to Hz.
                                    num2 *= 1000;
                                    if (s.charAt(0) == 'S') {
                                        // SSB
                                        fm.setLowFreqCutoff(num1);
                                        fm.setHighFreqCutoff(num2);
                                    } else {
                                        // CW or RTTY
                                        fm.setCenterFreq(num1);
                                        fm.setBandwidth(num2);
                                    }
                                }
                            }
                        }
                        if (s.indexOf("N G") != -1) {
                            int l = s.lastIndexOf(' ');
                            double gain = Double.valueOf(s.substring(l + 1));
                            fm.setDenoiserGain(gain);
                        }
                        if (s.indexOf("F G") != -1) {
                            int l = s.lastIndexOf(' ');
                            double gain = Double.valueOf(s.substring(l + 1));
                            fm.setFilterGain(gain);
                        }
                        if (s.indexOf("DSP") == 0) {
                            fm.setDspBypass(s.indexOf("ON") != -1);
                        }
                        if (s.indexOf("NR") == 0) {
                            int i = s.indexOf(' ');
                            int l = s.length();
                            fm.setNrBeta(Integer.valueOf(s.substring(i + 1, i + 3)));
                            fm.setNrHold(Integer.valueOf(s.substring(l - 2, l)));
                        }
                        if (s.indexOf("NT") == 0) {
                            int i = s.indexOf(' ');
                            int l = s.length();
                            fm.setNtBeta(Integer.valueOf(s.substring(i + 1, i + 3)));
                            fm.setNtHold(Integer.valueOf(s.substring(l - 2, l)));
                        }
                        varVal("SW17;", false, 100);
                    }
                    s = dec.getDisplay(sport, true);
                    cycles++;
                    if (cycles >= 6) {
                        break;
                    }
                } while (!first.equals(s));
                afil[m][filter - 1] = fm;
                varVal("SW29;", false); // AFIL, move to next DSP xfil.
            }
        }
        varVal("MD3;", false); // Leave in CW mode.
    }

    public String varVal(String command, boolean getDisplay) {
        sport.serialWrite(command.getBytes());
        byte response[] = new byte[16];
        sport.serialRead(response);
        if (getDisplay) {
            // Get response from rig and decode it.
            return dec.getDisplay(sport, true);
        }
        return null;
    }

    public String varVal(String command, boolean getDisplay, int delay) {
        sport.serialWrite(command.getBytes());
        byte response[] = new byte[16];
        sport.serialRead(response);
        sleep(delay);
        if (getDisplay) {
            // Get response from rig and decode it.
            return dec.getDisplay(sport, true);
        }
        return null;
    }

    public void sleep(int msec) {
        try {
            Thread.sleep(msec);
        } catch (InterruptedException e) {
        }
    }

    public JPopupMenu serialPopupChoices(final Main main) {
        String serialPorts[] = sport.getSerialPortNames();
        Arrays.sort(serialPorts);
        JMenuItem menuItem;
        JPopupMenu popup = new JPopupMenu();
        JLabel name = new JLabel("Choose K2 serial port");
        popup.add(name);
        popup.addSeparator();
        for (int i = 0; i < serialPorts.length; i++) {
            menuItem = new JMenuItem(serialPorts[i]);
            menuItem.addMouseListener(new MouseAdapter() {

                @Override
                public void mousePressed(MouseEvent e) {
                    String port = ((JMenuItem) e.getSource()).getText();
                    sport.setSerialPort("K2", port);
                    if (initialize()) {
                        Config c = new Config();
//                        c.readConfigFile();
                        main.enableModes(true);
                        main.enableDsps(true);
                        if (!c.filtersOk()) {
                            main.setDspVersion(kdsp2Version());
                            main.setCtlVersion(kdsp2CtlVersion());
                            getK2AllCrystalSettings();
                            getDSPSettings();
                            c.setCrystalFilters(xfil);
                            c.setDspFilters(afil);
                            c.writeConfigFile();
                        }
                        main.setCrystalFilters(xfil);
                        main.setDspFilters(afil);
                    }
                }
            });
            popup.add(menuItem);
        }
        return popup;
    }
}
