package util;

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import java.io.*;
import java.util.*;
//import gnu.io.*;

public class Serial {

    CommPortIdentifier portId;
    InputStream inputStream;
    OutputStream outputStream;
    static Enumeration portList;

    public Serial() {
        inputStream = null;
        outputStream = null;
    }

    public void setSerialPort(String app, String portName) {
        portId = findSerialPort(portName);
        serialSetup(app, portId);
    }

    public CommPortIdentifier findSerialPort(String portName) {

        portId = null;

        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL
                    && portId.getName().equals(portName)) {
                return portId;
            }
        }
        System.out.println("Port " + portName + " not found.");
        return null;
    }

    public String[] getSerialPortNames() {

        // Count number of serial ports.
        int numSports = 0;
        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                numSports++;
            }
        }

        if (numSports == 0) {
            return null;
        }

        String sp[] = new String[numSports];
        int i = 0;
        portList = CommPortIdentifier.getPortIdentifiers();
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                sp[i++] = portId.getName();
            }
        }

        return sp;
    }

    private void serialSetup(String app, CommPortIdentifier portId) {

        SerialPort serialPort = null;

        try {
            serialPort = (SerialPort) portId.open(app, 2000);
        } catch (PortInUseException e) {
        }

        try {
            inputStream = serialPort.getInputStream();
            outputStream = serialPort.getOutputStream();
        } catch (IOException e) {
        }

        try {
            if (app.equals("K2")) {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft KIO2 manual.
                        4800,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_2,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } else if (app.equals("W1")) {
                serialPort.setSerialPortParams(
                        // Set up per Elecraft W1 manual.
                        9600,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            } else {
                System.out.println("Unrecognized app.");
            }

        } catch (UnsupportedCommOperationException e) {
        }

    }

    public int serialRead(byte[] response) {

        if (inputStream == null) {
            System.out.println("input strm, null");
            return 0;
        }

        int estimate = 0;
        try {
            estimate = inputStream.available();
        } catch (IOException e) {
        }

        int bytesRead = 0;
        if (estimate > 0) {
            try {
                bytesRead = inputStream.read(response);
            } catch (IOException e) {
            }
        } else {
            for (int i = 0; i < response.length; i++) {
                response[i] = (byte) ' ';
            }
        }

        return bytesRead;
    }

    public void serialWrite(byte[] cmd) {

        if (outputStream == null) {
            return;
        }

        try {
            outputStream.write(cmd);
        } catch (IOException e) {
        }
    }

    public void serialWrite(byte[] cmd, int delay) {

        if (outputStream == null) {
            return;
        }

        try {
            outputStream.write(cmd);
        } catch (IOException e) {
        }

        try {
            // Elecraft recommends a 100 msec wait after a write,
            // though they also say most commands will be processed
            // within 20 msec.
            Thread.sleep(delay);
        } catch (InterruptedException e) {
        }
    }

    public void serialWrite(String s) {
        serialWrite(s.getBytes());
    }

    public void serialWrite(String s, int delay) {
        serialWrite(s.getBytes(), delay);
    }
}
