/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package digital;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.Timer;
import w1.PowerMeter;
import w1.W1;

/**
 *
 * @author mm
 */
public class DigitalPanel extends JPanel implements PowerMeter {

    //static final int TIMEOUT=100; // milliseconds, timer setting.
    //static final int HOLD_DELAY=2; // tap < HOLD_DELAY*TIMEOUTS < hold
    //static final int MSRT_DELAY=1; // MSRT_DELAY*TIMEOUTS < fwd/refl/swr cmd
    static final int TIMEOUT = 120; // milliseconds, timer setting.
    static final int HOLD_DELAY = 1; // tap < HOLD_DELAY*TIMEOUTS < hold
    static final int MSRT_DELAY = 1; // MSRT_DELAY*TIMEOUTS < fwd/refl/swr cmd
    private double fwd_W, rfl_W, max, nextF, nextR;
    private Font smallFont;
    private Image meterFace;
    static String w1pic = "images/w1panel.png";
    private String fw;
    private VUMeter fwdVuMeter, rflVuMeter, swrVuMeter;
    private W1State ws;
    int timerDivider;
    // Timer used to distinguish between button taps and holds, as well
    // as retrieve display updates over the serial link from the real W1.
    private Timer holdTimer;
    private Button button[], pushedButton;
    private W1 w1;

    /**
     * Creates new form DigitalPanel
     */
    public DigitalPanel() {
        this(null);
    }

    public DigitalPanel(W1 w1) {
        initComponents();
        this.w1 = w1;

        fw = null; // Firmware version.
        nextF = nextR = -1.;
        smallFont = new Font("SansSerif", Font.BOLD, 10);
        // Set up serial port.
        // Set up VU meters
        fwdVuMeter = new VUMeter("FWD", 88, 46, 247, 16);
        fwdVuMeter.setMin(0);
        fwdVuMeter.setUnits(" W");
        fwdVuMeter.setLabelX(44);
        fwdVuMeter.setLabelY(60);
        fwdVuMeter.setUnitsLabelX(338);
        fwdVuMeter.setUnitsLabelY(60);
        rflVuMeter = new VUMeter("REFL", 88, 68, 247, 16);
        rflVuMeter.setMin(0);
        rflVuMeter.setUnits(" W");
        rflVuMeter.setLabelX(44);
        rflVuMeter.setLabelY(82);
        rflVuMeter.setUnitsLabelX(338);
        rflVuMeter.setUnitsLabelY(83);
        swrVuMeter = new VUMeter("SWR", 88, 90, 247, 16);
        swrVuMeter.setMin(1);
        swrVuMeter.setMax(10);
        swrVuMeter.setUnits("");
        swrVuMeter.setLabelX(44);
        swrVuMeter.setLabelY(104);
        swrVuMeter.setUnitsLabelX(338);
        swrVuMeter.setUnitsLabelY(104);
        // Set up initial W1 state.
        ws = new W1State();
        ws.setRangeLocation(88, 125);
        ws.setDropLocation(146, 125);
        ws.setPwrLocation(196, 125);
        ws.setPeakLocation(242, 125);
        ws.setDecayLocation(303, 125);
        ws.setLEDLocation(355, 125);
        ws.setRange("AUTO");

        button = mkButtons();
        pushedButton = null;

        timerDivider = 0; // Used to count (divide) timer intervals.
    }

    @Override
    public void setDecayRate(char decayRate) {
        fwdVuMeter.setDecayRate(decayRate);
        rflVuMeter.setDecayRate(decayRate);
        ws.setDecay(decayRate);

    }

    @Override
    public void setFirmwareVersion(String fw) {
        this.fw = fw;
    }

    @Override
    public void setFwd_W(double fwd_W) {
        this.fwd_W = fwd_W;
        fwdVuMeter.setMeasurement(fwd_W); // Update vu meter.
        if (ws.getRangeMax() == 0) { // W1 is set to auto-range.
            nextF = ws.getNextRangeMax(fwd_W);
            if (nextR != -1.) {
                max = (nextF > nextR) ? nextF : nextR;
                fwdVuMeter.setNextMax(max);
                rflVuMeter.setNextMax(max);
                nextR = nextF = -1.;
            }
        }
    }

    @Override
    public boolean isLedsOn() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setLedsOn(boolean ledsOn) {
        ws.setLED(ledsOn ? "ON " : "OFF");
    }

    @Override
    public boolean isMonochrome() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setMonochrome(boolean monochrome) {
    }

    @Override
    public boolean isPeakHold() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setPeakHold(boolean hold) {
        char holdStr = hold ? 'H' : 'R';
        ws.setPeak(holdStr);
        fwdVuMeter.setPeakHold(hold ? 'O' : 'N');
        rflVuMeter.setPeakHold(hold ? 'O' : 'N');
    }

    @Override
    public boolean isPEP() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setPEP(boolean pep) {
        ws.setPwr(pep ? 'P' : 'A');
    }

    @Override
    public void setRfl_W(double rfl_W) {
        this.rfl_W = rfl_W;
        // Set vu-meter to pwrRefl
        rflVuMeter.setMeasurement(rfl_W);
        if (ws.getRangeMax() == 0) { // W1 is set to auto-range.
            nextR = ws.getNextRangeMax(rfl_W);
            if (nextF != -1.) {
                // Update auto-range when both fwd and refl
                // readings have been taken.
                max = (nextF > nextR) ? nextF : nextR;
                fwdVuMeter.setNextMax(max);
                rflVuMeter.setNextMax(max);
                nextR = nextF = -1.;
            }
        }
    }

    @Override
    public void startFrameTimer() {

        /**
         * This is called each time the swing timer goes off. It handles button
         * holds, not taps. And if the timer goes off when there was not a
         * button hold, then the latest fordwa, reflected, and swr readings are
         * retrieved from the W1.
         */
        holdTimer = new Timer(TIMEOUT, new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                //holdTimer.stop();
                timerDivider++; // Another timer interval counted.
                if (timerDivider >= HOLD_DELAY && pushedButton != null) {
                    // Button's been held.
                    String bf = pushedButton.getHoldFn();
                    ws.setLabelSecondary(pushedButton.getName(),
                            pushedButton.getSecondaryLabel());
                    if (bf == "") {
                        System.exit(0); // On/off button hold exits.
                    }
                    w1.w1cmd(bf);
                    pushedButton = null;
                } else if (timerDivider >= MSRT_DELAY) {
                    // Get latest measurements for fwd, refl, swr.
                    //w1cmd("FR");
                    w1.w1cmd("FR");
                    // Calculate SWR, don't waste time retrieving it from W1.
                    if (fwd_W == 0) {
                        swrVuMeter.setMeasurement(1);
                    } else {
                        double g = Math.sqrt(rfl_W / fwd_W);
                        swrVuMeter.setMeasurement((1 + g) / (1 - g));
                    }
                    if (timerDivider >= HOLD_DELAY) // Just so timer doesn't overflow...
                    {
                        timerDivider = 0;
                    }
                }
                //holdTimer.start();
                repaint();
            }
        });
        holdTimer.setRepeats(true);
        holdTimer.start();
    }

    @Override
    public void stopFrameTimer() {
        if (holdTimer != null && holdTimer.isRunning()) {
            holdTimer.stop();
        }
    }

    @Override
    public void setDropRate(char dropRate) {
        fwdVuMeter.setDropRate(dropRate);
        rflVuMeter.setDropRate(dropRate);
        ws.setDrop(dropRate);
    }

    @Override
    public void setRange(char range) {
        ws.setRange(range);
        max = ws.getRangeMax();
        fwdVuMeter.setMax(max);
        rflVuMeter.setMax(max);
    }

    @Override
    public boolean isUnits_W() {
        return fwdVuMeter.isUnits_W();
    }

    @Override
    public void setUnits_W(boolean units_W) {
        fwdVuMeter.setUnits_W(units_W);
        rflVuMeter.setUnits_W(units_W);
    }

    /**
     * Create the W1 buttons.
     */
    public Button[] mkButtons() {
        Button b[] = new Button[5];
        b[0] = new ButtonRcl("rcl/store", 76, 188, 46, 22);
        b[1] = new ButtonRange("range/drop", 138, 188, 46, 22);
        b[2] = new ButtonPwr("pwr/peak", 202, 188, 46, 22);
        b[3] = new ButtonDecay("decay/led", 264, 188, 46, 22);
        b[4] = new ButtonOff("on/off", 328, 188, 46, 22);
//        buttonRange = (ButtonRange) b[1];
//        buttonPwr = (ButtonPwr) b[2];
//        buttonDecay = (ButtonDecay) b[3];
        return b;
    }

    /**
     * NOTE: taken from SimpleBeanInfo.java<p>
     *
     * This is a utility method to help in loading icon images. It takes the
     * name of a resource file associated with the current object's class file
     * and loads an image object from that file. Typically images will be GIFs.
     * <p>
     *
     * @param resourceName A pathname relative to the directory holding the
     * class file of the current class. For example, "wombat.gif".
     * @return an image object. May be null if the load failed.
     */
    public java.awt.Image loadImage(final String resourceName) {
        try {
            final Class c = getClass();
            java.awt.image.ImageProducer ip = (java.awt.image.ImageProducer) java.security.AccessController.doPrivileged(
                    new java.security.PrivilegedAction() {

                @Override
                        public Object run() {
                            java.net.URL url;
                            if ((url = c.getResource(resourceName)) == null) {
                                return null;
                            } else {
                                try {
                                    return url.getContent();
                                } catch (java.io.IOException ioe) {
                                    return null;
                                }
                            }
                        }
                    });

            if (ip == null) {
                return null;
            }
            java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
            return tk.createImage(ip);
        } catch (Exception ex) {
            return null;
        }
    }

    /**
     * Paint the meter on the screen.
     */
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (meterFace == null) {
            meterFace = loadImage("images/w1panel.png");
        }
        g.drawImage(meterFace, 0, 0, this);
        fwdVuMeter.display(g);
        rflVuMeter.display(g);
        swrVuMeter.display(g);
        ws.display(g);
        if (fw != null) {
            g.setColor(Color.white);
            g.setFont(smallFont);
            g.drawString(fw, 20, 257);
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setInheritsPopupMenu(true);
        setPreferredSize(new java.awt.Dimension(452, 267));
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mousePressedHandler(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mouseReleasedHandler(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void mousePressedHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mousePressedHandler
        /**
         * Either bring up serial port pop up menu or record which button was
         * pushed, if any, and restart timer to later determine if a button tap
         * or hold took place.
         */
        int x = evt.getX();
        int y = evt.getY();

        if (evt.isPopupTrigger()) {
            JPopupMenu popup = w1.mkPopUpMenu();
            popup.show(this, x, y);
        }

        // See if a button was pushed.
        for (int i = 0; i < button.length; i++) {
            if (button[i].contains(x, y)) {
                pushedButton = button[i];
            }
        }
        // Start timer to see if user performs button tap or hold.
        timerDivider = 0;
        holdTimer.restart();

    }//GEN-LAST:event_mousePressedHandler

    private void mouseReleasedHandler(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseReleasedHandler
        // If timer is still running, this is a button "tap"
        // event so handle it in this method.  If not, a
        // longer button "hold" has already been handled
        // by holdPerformer() before this release.

        if (evt.isPopupTrigger()) {
            JPopupMenu popup = w1.mkPopUpMenu();
            popup.show(this, evt.getX(), evt.getY());
        }


        if (pushedButton != null) {
            String bf = pushedButton.getTapFn();
            ws.setLabelPrimary(pushedButton.getName(),
                    pushedButton.getPrimaryLabel());
            w1.w1cmd(bf);
            pushedButton = null;
        }
    }//GEN-LAST:event_mouseReleasedHandler
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
